﻿//////////////////////////////////////////////
// PbsProgramTracker.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkAstraeus
{
	struct PbsMaterialInfo ;
}

namespace nkGraphics
{
	class Program ;
}

/// Includes ---------------------------------

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkGraphics
#include <NilkinsGraphics/Programs/Program.h>
#include <NilkinsGraphics/Programs/ProgramManager.h>

// Standards
#include <unordered_map>

/// Internals --------------------------------

namespace nkAstraeus
{
	struct ProgramEntry
	{
		nkGraphics::Program* _program = nullptr ;
		unsigned int _count = 0 ;
	} ;
}

/// Class ------------------------------------

namespace nkAstraeus
{
	template <typename T_info, typename T_idGenerator, typename T_nameBase>
	class ProgramTracker
	{
		protected :

			// Attributes
			// Memory of programs
			std::unordered_map<unsigned long long, ProgramEntry> _programs ;

		protected :

			// Functions
			// Constructor, destructor
			ProgramTracker () = default ;

			~ProgramTracker ()
			{
				// Free of existing programs
				for (const std::pair<const unsigned long long, ProgramEntry>& entry : _programs)
					nkGraphics::ProgramManager::getInstance()->erase(entry.second._program->getResourceName()) ;

				// Clear existing memory
				_programs.clear() ;
			}

		public :

			// Management
			nkGraphics::Program* getProgram (const T_info& info)
			{
				// Find back the index first
				unsigned long long index = T_idGenerator::generate(info) ;

				std::unordered_map<unsigned long long, ProgramEntry>::iterator searchResult = _programs.find(index) ;

				if (searchResult != _programs.end())
				{
					// Increment count and return
					searchResult->second._count++ ;
					return searchResult->second._program ;
				}

				// Need to create it
				std::string programName (T_nameBase::name) ;
				programName += std::to_string(index) ;

				nkGraphics::Program* result = nkGraphics::ProgramManager::getInstance()->createOrRetrieve(programName) ;
				result->setHidden(true) ;

				// Register
				_programs.emplace(index, ProgramEntry{result, 1}) ;

				// And return this new entry
				return result ;
			}

			void releaseProgram (const T_info& info)
			{
				unsigned long long index = T_idGenerator::generate(info) ;

				std::unordered_map<unsigned long long, ProgramEntry>::iterator searchResult = _programs.find(index) ;

				if (searchResult != _programs.end())
				{
					// Decrement and check value
					--searchResult->second._count ;

					if (!searchResult->second._count)
					{
						// Need to free then, both in nkGraphics and our memory
						nkGraphics::ProgramManager::getInstance()->erase(searchResult->second._program->getResourceName()) ;
						_programs.erase(searchResult) ;
					}

				}
			}
	} ;
}