﻿//////////////////////////////////////////////
// BufferManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Buffer ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{	
	class DLL_GRAPHICS_EXPORT BufferManager : public nkCommon::SingletonClass<BufferManager>
	{
		friend class nkCommon::SingletonClass<BufferManager> ;

		private :
		
			// Attributs
			// Liste des buffers en memoire
			std::unordered_map<std::string, Buffer*> _memory ;

			// Threading
			std::mutex _mutex ;

		private :

			// Constructeur
			BufferManager () ;
			
		public :

			// Destructeur
			~BufferManager () ;
		
			// Manipulation mémoire
			Buffer* createOrRetrieve (const std::string_view& name) ;
			Buffer* get (const std::string_view& name) ;
			Buffer* getByIndex (unsigned int index) ;
			void rename (const std::string_view& oldName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;
	} ;
}