﻿//////////////////////////////////////////////
// Camera.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "Frustum.h"

#include "../Dll/DllDefines.h"

#include "../Graph/Node.h"

// nkExport
#include <NilkinsExport\Exportable\Exportable.h>

// nkMaths
#include <NilkinsMaths/Algebra/Vector.h>
#include <NilkinsMaths/Algebra/Quaternion.h>
#include <NilkinsMaths/Algebra/Matrix.h>

// Standards
#include <string>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Camera : public Node
	{
		private :
		
			// Attributs
			// Nom de la camera pour la retrouver facilement
			std::string _name ;

			// Sur le rendu
			nkMaths::Matrix _viewMat ;
			nkMaths::Matrix _projMat ;

			// Pour définir les clips
			float _near ;
			float _far ;
			float _aspectRatio ;
			float _fov ;

			// Frustum actif
			Frustum _frustum ;
			bool _dirtyFrustum ;

			// Flags
			bool _autoUpdateOnContextSwitch ;

			// Logique un peu plus resource
			bool _hidden ;
		
		public :
		
			// Constructeur, destructeur
			Camera () ;
			~Camera () ;

			// Getters
			float getNear () const ;
			float getFar () const ;
			Frustum& getFrustum () ;
			std::string_view getName () const ;
			float getFov () const ;
			float getAspectRatio () const ;
			bool getAutoUpdateOnContextSwitch () const ;
			bool getHidden () const ;
			const nkMaths::Matrix& getViewMatrix () const ;
			const nkMaths::Matrix& getProjectionMatrix () const ;
			nkMaths::Matrix getViewProjMatrix () const ;

			// Setters
			virtual void setPositionRelative (const nkMaths::Vector& value) override ;
			virtual void setPositionAbsolute (const nkMaths::Vector& value) override ;
			virtual void translateRelative (const nkMaths::Vector& value) override ;
			virtual void translateAbsolute (const nkMaths::Vector& value) override ;
			virtual void setOrientationRelative (const nkMaths::Quaternion& value) override ;
			virtual void setOrientationAbsolute (const nkMaths::Quaternion& value) override ;
			virtual void rotateRelative (const nkMaths::Quaternion& value) override ;
			virtual void rotateAbsolute (const nkMaths::Quaternion& value) override ;
			virtual void setScaleRelative (const nkMaths::Vector& value) override ;
			virtual void setScaleAbsolute (const nkMaths::Vector& value) override ;
			virtual void applyScale (const nkMaths::Vector& value) override ;
			void setNear (float value) ;
			void setFar (float value) ;
			void setName (const std::string_view& name) ;
			void setFov (float radianValue) ;
			void setAspectRatio (float widthOnHeight) ;
			void prepareForContext (RenderContext* context) ;
			void setAutoUpdateOnContextSwitch (bool value) ;
			void setHidden (bool value) ;

			// Recalcul matrice de vue
			void updateViewMatrix () ;
			void updateProjectionMatrix () ;
			void lookAt (const nkMaths::Vector& point, const nkMaths::Vector& enforcedUp = nkMaths::Vector(0, 0, 0, 0)) ;

			// Obtention infos utiles
			void getNearPlanePoints (nkMaths::Vector& a, nkMaths::Vector& b, nkMaths::Vector& c, nkMaths::Vector& d) ;
			void getFarPlanePoints (nkMaths::Vector& a, nkMaths::Vector& b, nkMaths::Vector& c, nkMaths::Vector& d) ;
			void getFrustumCornersDirectionsWorld (nkMaths::Vector& a, nkMaths::Vector& b, nkMaths::Vector& c, nkMaths::Vector& d) ;
			void getFrustumCornersDirectionsView (nkMaths::Vector& a, nkMaths::Vector& b, nkMaths::Vector& c, nkMaths::Vector& d) ;

			// Raycast
			nkMaths::Vector getDirectionAtPixelWorld (int x, int y, RenderContext* fromContext = nullptr) ;
			nkMaths::Vector getDirectionAtScreenCoordWorld (float x, float y) ;
			nkMaths::Vector getDirectionAtPixelView (int x, int y, RenderContext* fromContext = nullptr) ;
			nkMaths::Vector getDirectionAtScreenCoordView (float x, float y) ;
				
			// Les fonctions d'Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) ;
			virtual void importClassFromTree (nkExport::Node* rootNode) ;
	} ;
}