﻿//////////////////////////////////////////////
// Program.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class ProgramSourcesHolder ;
	class StreamOutputDescriptor ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Resources/Resource.h"

#include "../Utilities/DxDefinesWrapper.h"
#include "../Utilities/InfiniteMask.h"

#include "ProgramCompilationResult.h"
#include "ProgramType.h"

// Export
#include <NilkinsExport/Exportable/Exportable.h>

// Standards
#include <vector>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Program : public Resource, public nkExport::Exportable
	{
		protected :

			// Attributs
			// Le résultat de la dernière compilation
			ProgramCompilationResult _lastCompilationResult ;

			// Mémoire
			ProgramSourcesHolder* _memory ;

			// Les sémantiques que nous avons en mémoire, avec leurs indices de registre
			InfiniteByteMask _heldSemantics ;

			// Id pour sorting
			unsigned int _id ;

			// Type de shader holder
			PROGRAM_TYPE _type ;
			// Profil de compilation choisi
			PROGRAM_COMPILATION_PROFILE _profile ;

			// Témoin d'origine
			bool _fromFile ;
			bool _filesLoaded ;
			bool _fromMemory ;
		
		public :
		
			// Constructeur, destructeur
			Program () ;
			virtual ~Program () ;

			// Getters
			const ProgramCompilationResult& getLastCompilationResult () const ;
			bool isFromFile () const ;
			bool isFromMemory () const ;
			ProgramSourcesHolder* getMemory () const ;
			const InfiniteByteMask& getSemantics () const ;
			unsigned int getId () const ;
			PROGRAM_TYPE getType () const ;
			PROGRAM_COMPILATION_PROFILE getProfile () const ;

			// Setters
			virtual void setFromMemory (const ProgramSourcesHolder& memory) ;
			virtual void setDefines (const ProgramSourcesHolder& memory) ;
			void setFromFiles (const char* folder) ;
			void setId (unsigned int id) ;
			void setProfile (PROGRAM_COMPILATION_PROFILE profile) ;

			// Fonctions de chargement
			bool prepareMemoryWithFiles () ;
			virtual bool loadFromMemory () = 0 ;
			virtual bool tryToLoadFrom (ProgramSourcesHolder& newMemory) = 0 ;

			// Load brut
			virtual bool load () override ;
			virtual void unload () override ;			

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}