﻿//////////////////////////////////////////////
// GlesTexture.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

typedef unsigned int GLenum ;
typedef unsigned int GLuint ;

/// Includes ---------------------------------

// nkGraphics
#include "../../../Textures/Texture.h"

/// Class ------------------------------------

namespace nkGraphics::gles
{
	class GlesTexture final : public Texture
	{
		private :

			// Attributes
			// Id
			GLuint _id ;

			// Size
			unsigned int _width ;
			unsigned int _height ;
			unsigned int _depthOrArraySize ;

			// Formats
			GLenum _internalFormat ;
			GLenum _format ;
			GLenum _formatType ;

		public :

			// Constructor, destructor
			GlesTexture () ;
			~GlesTexture () ;

			// Getters
			virtual int getWidth () const override ;
			virtual int getHeight () const override ;
			virtual int getDepthOrArraySize () const override ;
			virtual unsigned int getMipLevels () const override ;
			virtual FORMAT getTextureFormat () const override ;
			virtual CPU_ACCESS_FLAG getCpuAccessRights () const override ;
			virtual USAGE getTextureUsage () const override ;
			virtual BIND_FLAG getTextureBinding () const override ;
			virtual RESOURCE_MISC_FLAG getMiscFlag () const override ;
			virtual unsigned int getSampleCount () const override ;
			virtual unsigned int getSampleQuality () const override ;
			virtual bool getAutoMipGeneration () const override ;
			GLuint getId () const ;

			// Setters
			virtual void setWidth (unsigned int width) override ;
			virtual void setHeight (unsigned int height) override ;
			virtual void setDepth (unsigned int depth) override ;
			virtual void setArraySize (unsigned int size) override ;
			virtual void setMipLevels (unsigned int value) override ;
			virtual void setTextureFormat (FORMAT format) override ;
			virtual void setCpuAccessRights (CPU_ACCESS_FLAG rights) override ;
			virtual void setTextureUsage (USAGE usage) override ;
			virtual void setTextureBinding (BIND_FLAG binding) override ;
			virtual void setAutoMipGeneration (bool value) override ;
			virtual void copyParams (const Texture& other) override ;

			// Load
			virtual bool preLoadFromFile () override ;
			virtual bool load () override ;
			virtual void unload () override ;

			// Activation
			virtual void activate (SHADER_TYPE shaderStage, unsigned int slot) override ;

			// Mapping
			virtual MapResult map (const MapRequestDescriptor& requestDesc) override ;
			virtual void unmap (const UnmapRequestDescriptor& requestDesc) override ;
			virtual void gpuCopyFrom (Texture& other, const ResourceGpuCopyDescriptor& descriptor) override ;
	} ;
}